import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.math.BigDecimal;

public class Hanoi4
{
  private int disks;
  private static int moves;

  public Hanoi4(int disks)
  {
    this.disks = disks;
  }

  public void doIt()
  {
    if (1==this.disks) 
    {
      System.out.println("1->2");
      Hanoi4.moves++;
      return;
    }
    if (2==disks) 
    {
      System.out.println("1->3");
      Hanoi4.moves++;
      System.out.println("1->2");
      Hanoi4.moves++;
      System.out.println("3->2");
      Hanoi4.moves++;
      return;
    }
    if (3==disks) 
    {
      System.out.println("1->3");
      Hanoi4.moves++;
      System.out.println("1->4");
      Hanoi4.moves++;
      System.out.println("1->2");
      Hanoi4.moves++;
      System.out.println("4->2");
      Hanoi4.moves++;
      System.out.println("3->2");
      Hanoi4.moves++;
      return;
    }
    int k = calcMin(this.disks);
//    System.out.println("k ="+k);
    hanoi4(1, 4, 2, 3, k);
    hanoi3(1, 2, 3, this.disks-k);
    hanoi4(4, 2, 1, 3, k);
  }

  private int calcMin(int disks)
  {
    BigDecimal[] H4 = new BigDecimal[disks+1];
    int[] k_H4 = new int[disks+1];
    if (0>=disks)
    {
      System.err.println("FATAL ERROR: # of disks is <= 0");
      Runtime.getRuntime().exit(-1);
    }
    if (1==disks)
    {
      return 0; // H4(1)=1, k = 0
    }
    if (2==disks)
    {
      return 1; // H4(2)=3, k = 1
    }
    H4[1] = new BigDecimal("1"); k_H4[1]=0;
    H4[2] = new BigDecimal("3"); k_H4[2]=1;
    for (int n = 3; n<=disks; n++)
    {
      BigDecimal min = new BigDecimal("0");
      int min_k = 0;
      for (int k = 1; k<n; k++)
      {
        if (1==k)
	{
          min = H4[1].multiply(new BigDecimal("2"));
          BigDecimal p = new BigDecimal("2");
          p = p.pow(n-1);
	  min = min.add(p);
	  min = min.subtract(new BigDecimal("1"));
	  min_k = 1;
//        if (n==disks)
//	  {
//            System.out.println("k ="+k+" : "+min.toString());
//	  }
	}
	else
	{
          BigDecimal newMin = H4[k].multiply(new BigDecimal("2"));
          BigDecimal p = new BigDecimal("2");
          p = p.pow(n-k);
	  newMin = newMin.add(p);
	  newMin = newMin.subtract(new BigDecimal("1"));
          if (min.compareTo(newMin)>0)
	  {
            min = newMin;
            min_k = k;
	  }
//        if (n==disks)
//	  {
//            System.out.println("k ="+k+" : "+newMin.toString());
//	  }
	}
      }
      H4[n]=min;
      k_H4[n]=min_k;
    }
    return k_H4[disks];
  }

  public void hanoi3(int from, int to, int using, int disks)
  {
    if (disks<=0) return;
    if (disks>1) hanoi3(from, using, to, disks-1);
    System.out.println(from+"->"+to);
    Hanoi4.moves++;
    if (disks>1) hanoi3(using, to, from, disks-1);
  }

  public void hanoi4(int from, int to, int using1, int using2, int disks)
  {
    if (1==disks) 
    {
      System.out.println(from+"->"+to);
      Hanoi4.moves++;
      return;
    }
    if (2==disks) 
    {
      System.out.println(from+"->"+using1);
      Hanoi4.moves++;
      System.out.println(from+"->"+to);
      Hanoi4.moves++;
      System.out.println(using1+"->"+to);
      Hanoi4.moves++;
      return;
    }
    if (3==disks) 
    {
      System.out.println(from+"->"+using1);
      Hanoi4.moves++;
      System.out.println(from+"->"+using2);
      Hanoi4.moves++;
      System.out.println(from+"->"+to);
      Hanoi4.moves++;
      System.out.println(using2+"->"+to);
      Hanoi4.moves++;
      System.out.println(using1+"->"+to);
      Hanoi4.moves++;
      return;
    }
    int k = calcMin(disks);
//    System.out.println("k ="+k);
    hanoi4(from, using2, to, using1, k);
    hanoi3(from, to, using1, disks-k);
    hanoi4(using2, to, from, using1, k);
  }

  public static void main(String args[])
  {
    BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
    System.out.print("Input the number of disks :");
    try
    {
      int disks = Integer.parseInt(in.readLine());
      if (0>=disks)
      {
        System.err.println("# of disks is <= 0");
	return;
      }
      Hanoi4 hanoi = new Hanoi4(disks);
      Hanoi4.moves = 0;
      hanoi.doIt();
      System.out.print("Total moves :"+Hanoi4.moves);
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }
}

